@@ -17,20 +17,24 @@ mod num;
 mod primitive_types;
 #[cfg(feature = "usize")]
 mod string;
+mod tuple;
 #[cfg(feature = "usize")]
 mod vec;
 
+use alloc::vec::Vec;
+use core::{
+    convert::{AsRef, Infallible},
+    fmt::Debug,
+};
+
+pub use packable_derive::Packable;
+
 use crate::{
     error::{UnexpectedEOF, UnpackError},
     packer::{LenPacker, Packer},
     unpacker::Unpacker,
 };
 
-pub use packable_derive::Packable;
-
-use alloc::vec::Vec;
-use core::{convert::AsRef, fmt::Debug};
-
 /// A type that can be packed and unpacked.
 ///
 /// Almost all basic sized types implement this trait. This trait can be derived using the
@@ -40,10 +44,11 @@ use core::{convert::AsRef, fmt::Debug};
 ///
 /// We will implement [`Packable`] for a type that encapsulates optional integer values (like `Option<i32>`).
 ///
-/// Following the conventions from the [IOTA protocol messages RFC](https:///github.com/iotaledger/protocol-rfcs/pull/0017),
-/// we will use an integer prefix as a tag to determine which variant of the enum is being packed.
+/// We will use an integer prefix as a tag to determine which variant of the enum is being packed.
 ///
 /// ```rust
+/// use core::convert::Infallible;
+///
 /// use packable::{
 ///     error::{UnknownTagError, UnpackError, UnpackErrorExt},
 ///     packer::Packer,
@@ -51,8 +56,6 @@ use core::{convert::AsRef, fmt::Debug};
 ///     Packable,
 /// };
 ///
-/// use core::convert::Infallible;
-///
 /// pub enum Maybe {
 ///     Nothing,
 ///     Just(i32),
@@ -76,7 +79,7 @@ use core::{convert::AsRef, fmt::Debug};
 ///     fn unpack<U: Unpacker, const VERIFY: bool>(
 ///         unpacker: &mut U,
 ///     ) -> Result<Self, UnpackError<Self::UnpackError, U::Error>> {
-///         match u8::unpack::<_, VERIFY>(unpacker).infallible()? {
+///         match u8::unpack::<_, VERIFY>(unpacker).coerce()? {
 ///             0u8 => Ok(Self::Nothing),
 ///             1u8 => Ok(Self::Just(i32::unpack::<_, VERIFY>(unpacker).coerce()?)),
 ///             tag => Err(UnpackError::Packable(UnknownTagError(tag))),
@@ -169,12 +172,12 @@ use core::{convert::AsRef, fmt::Debug};
 /// where `F` is the type of the field being verified, `P` is the type of the `struct` or `enum`
 /// and `VERIFY` is the same constant parameter used inside `Packable::unpack`. This verification
 /// function will be run immediately after unpacking the field.
-pub trait Packable: Sized {
+pub trait Packable: Sized + 'static {
     /// The error type that can be returned if some semantic error occurs while unpacking.
     ///
     /// It is recommended to use [`Infallible`](core::convert::Infallible) if this kind of error is impossible or
     /// [`UnknownTagError`](crate::error::UnknownTagError) when implementing this trait for an enum.
-    type UnpackError: Debug;
+    type UnpackError: Debug + From<Infallible>;
 
     /// Packs this value into the given [`Packer`].
     fn pack<P: Packer>(&self, packer: &mut P) -> Result<(), P::Error>;
@@ -207,6 +210,7 @@ pub trait PackableExt: Packable {
 }
 
 impl<P: Packable> PackableExt for P {
+    #[inline]
     fn packed_len(&self) -> usize {
         let mut packer = LenPacker(0);
 
@@ -216,6 +220,7 @@ impl<P: Packable> PackableExt for P {
         }
     }
 
+    #[inline]
     fn pack_to_vec(&self) -> Vec<u8> {
         let mut packer = Vec::with_capacity(self.packed_len());
 
@@ -226,6 +231,7 @@ impl<P: Packable> PackableExt for P {
     }
 
     /// Unpacks this value from a type that implements [`AsRef<[u8]>`].
+    #[inline]
     fn unpack_verified<T: AsRef<[u8]>>(
         bytes: T,
     ) -> Result<Self, UnpackError<<Self as Packable>::UnpackError, UnexpectedEOF>> {
@@ -233,6 +239,7 @@ impl<P: Packable> PackableExt for P {
     }
 
     /// Unpacks this value from a type that implements [`AsRef<[u8]>`] skipping some syntatical checks.
+    #[inline]
     fn unpack_unverified<T: AsRef<[u8]>>(
         bytes: T,
     ) -> Result<Self, UnpackError<<Self as Packable>::UnpackError, UnexpectedEOF>> {
